@@ -6,7 +6,7 @@ def get_classes():
 
     if (platforms.PLATFORM == platforms.MAEMO5):
         from RotationMonitor import RotationMonitor
-        classes.append(RotationMonitor)
+        #classes.append(RotationMonitor)
     #end if
     
     return classes
