@@ -53,6 +53,7 @@ class AudioAlbumStorage(AudioArtistStorage):
         f.resource = ""
         f.name = self.get_name()
         f.info = "Browse your music library by album"
+        f.icon = self.get_icon().get_path()
         f.folder_flags = f.ITEMS_ENQUEUEABLE | f.ITEMS_COMPACT
         
         return f
@@ -104,6 +105,7 @@ class AudioAlbumStorage(AudioArtistStorage):
             
             for filepath in index.list_files(album):
                 f = self.call_service(msgs.CORE_SVC_GET_FILE, filepath)
+                if (not f): continue
                 
                 tags = tagreader.get_tags(f)
                 f.name = tags.get("TITLE") or f.name
