@@ -25,12 +25,7 @@ class AudioGenreStorage(AudioArtistStorage):
     
         AudioArtistStorage.__init__(self)
 
-
-    def handle_message(self, msg, *args):
-    
-        pass
-
-        
+       
     def get_prefix(self):
     
         return "library://audio-genres"
@@ -53,9 +48,10 @@ class AudioGenreStorage(AudioArtistStorage):
         f.is_local = True
         f.can_skip = True
         f.path = "/"
-        f.mimetype = f.DIRECTORY
+        f.mimetype = f.DEVICE_ROOT
         f.resource = ""
         f.name = self.get_name()
+        f.icon = self.get_icon().get_path()
         f.info = "Browse your music library by genre"
         
         return f
