@@ -44,6 +44,7 @@ class Navigator(View):
 
         # now-playing box
         self.__now_playing_box = NowPlaying()
+        self.__now_playing_box.set_visible(False)
         self.add(self.__now_playing_box)
         self.__now_playing_box.connect_clicked(self.__on_click_now_playing)
 
@@ -59,10 +60,10 @@ class Navigator(View):
         self.__browser.connect_folder_begin(self.__on_begin_folder)
         self.__browser.connect_folder_progress(self.__on_progress_folder)
         self.__browser.connect_folder_complete(self.__on_complete_folder)
-        self.__browser.connect_file_added_to_library(self.__on_add_file_to_lib)
+        #self.__browser.connect_file_added_to_library(self.__on_add_file_to_lib)
         self.__browser.connect_file_opened(self.__on_open_file)
-        self.__browser.connect_file_enqueued(self.__on_enqueue_file)
-        self.__browser.connect_file_bookmarked(self.__on_bookmark_file)
+        #self.__browser.connect_file_enqueued(self.__on_enqueue_file)
+        #self.__browser.connect_file_bookmarked(self.__on_bookmark_file)
         self.__browser_slider.connect_button_pressed(
                                     lambda a,b:self.__browser.stop_scrolling())
         self.add(self.__browser)
@@ -83,6 +84,10 @@ class Navigator(View):
                                          theme.mb_btn_history_2)
         self.__btn_history.connect_clicked(self.__on_btn_history)
 
+        self.__btn_prefs = ImageButton(theme.mb_btn_prefs_1,
+                                       theme.mb_btn_prefs_2)
+        self.__btn_prefs.connect_clicked(self.__on_btn_prefs)
+
         self.__btn_bookmarks = ImageButton(theme.mb_btn_bookmark_1,
                                            theme.mb_btn_bookmark_2)
         self.__btn_bookmarks.connect_clicked(self.__on_btn_bookmarks)
@@ -118,7 +123,10 @@ class Navigator(View):
         """
 
         cwd = self.__browser.get_current_folder()
-        items = [self.__btn_home, self.__btn_history, self.__btn_back]
+        items = [self.__btn_home,
+                 self.__btn_history,
+                 self.__btn_prefs,
+                 self.__btn_back]
         
         if (cwd.folder_flags & cwd.ITEMS_ADDABLE):
             items.append(self.__btn_add)
@@ -208,7 +216,7 @@ class Navigator(View):
         if (self.is_visible()):
             self.__tn_scheduler.resume()
 
-
+    """
     def __on_add_file_to_lib(self, f):
     
         self.emit_message(msgs.LIBRARY_ACT_ADD_MEDIAROOT, f)
@@ -239,6 +247,7 @@ class Navigator(View):
     def __on_bookmark_file(self, f):
     
         f.bookmarked = True
+    """
 
 
     def __update_thumbnail(self, item, thumbpath):
@@ -278,7 +287,17 @@ class Navigator(View):
         
         f = self.call_service(msgs.CORE_SVC_GET_FILE, "history:///")
         if (f):
-            self.__browser.load_folder(f, self.__browser.GO_PARENT, True)
+            self.__browser.load_folder(f, self.__browser.GO_CHILD, True)
+
+
+    def __on_btn_prefs(self):
+        """
+        Reacts on pressing the [Prefs] button.
+        """
+        
+        f = self.call_service(msgs.CORE_SVC_GET_FILE, "preferences:///")
+        if (f):
+            self.__browser.load_folder(f, self.__browser.GO_CHILD, False)
 
 
     def __on_btn_bookmarks(self):
@@ -323,12 +342,17 @@ class Navigator(View):
         self.__current_file = f
         self.__browser.hilight_file(f)
 
-        if (not f.mimetype in mimetypes.get_image_types()):
-            self.emit_message(msgs.MEDIA_ACT_STOP)
+        if (f.mimetype == "application/x-applet"):
+            applet_id = f.resource
+            self.call_service(msgs.CORE_SVC_LAUNCH_APPLET, applet_id)
 
-        self.emit_message(msgs.UI_ACT_SELECT_VIEW, "MediaView")
-        self.emit_message(msgs.MEDIA_ACT_LOAD, f)
-        self.emit_message(msgs.MEDIA_EV_LOADED, self, f)
+        else:
+            if (not f.mimetype in mimetypes.get_image_types()):
+                self.emit_message(msgs.MEDIA_ACT_STOP)
+
+            self.emit_message(msgs.UI_ACT_SELECT_VIEW, "MediaView")
+            self.emit_message(msgs.MEDIA_ACT_LOAD, f)
+            self.emit_message(msgs.MEDIA_EV_LOADED, self, f)
 
 
     def render_this(self):
@@ -336,16 +360,26 @@ class Navigator(View):
         w, h = self.get_size()
         if (w < h):
             # portrait mode
-            self.__now_playing_box.set_geometry(0, 0, w, 80)
-            self.__browser_slider.set_geometry(0, 80, 40, h - 70 - 80)
-            self.__browser.set_geometry(40, 80, w - 40, h - 70 - 80)
+            if (self.__now_playing_box.is_visible()):
+                self.__now_playing_box.set_geometry(0, 0, w, 80)
+                self.__browser_slider.set_geometry(0, 80, 40, h - 70 - 80)
+                self.__browser.set_geometry(40, 80, w - 40, h - 70 - 80)
+            else:
+                self.__browser_slider.set_geometry(0, 0, 40, h - 70)
+                self.__browser.set_geometry(40, 0, w - 40, h - 70)
+            
             self.__toolbar.set_geometry(0, h - 70, w, 70)
 
         else:
             # landscape mode
-            self.__now_playing_box.set_geometry(0, h - 80, w - 70, 80)
-            self.__browser_slider.set_geometry(0, 0, 40, h - 80)
-            self.__browser.set_geometry(40, 0, w - 40 - 70, h - 80)
+            if (self.__now_playing_box.is_visible()):
+                self.__now_playing_box.set_geometry(0, h - 80, w - 70, 80)
+                self.__browser_slider.set_geometry(0, 0, 40, h - 80)
+                self.__browser.set_geometry(40, 0, w - 40 - 70, h - 80)
+            else:
+                self.__browser_slider.set_geometry(0, 0, 40, h)
+                self.__browser.set_geometry(40, 0, w - 40 - 70, h)
+
             self.__toolbar.set_geometry(w - 70, 0, 70, h)
 
 
@@ -487,6 +521,24 @@ class Navigator(View):
     
         icon, is_final = self.call_service(msgs.MEDIASCANNER_SVC_LOOKUP_THUMBNAIL, f)
         self.__now_playing_box.set_playing(icon, f)
+        self.__now_playing_box.set_action("Loaded:")
+        self.__now_playing_box.set_visible(True)
+        self.render()
+
+
+    def handle_MEDIA_EV_PLAY(self):
+    
+        self.__now_playing_box.set_action("Playing:")
+
+
+    def handle_MEDIA_EV_PAUSE(self):
+    
+        self.__now_playing_box.set_action("Paused:")
+
+
+    def handle_MEDIA_EV_EOF(self):
+    
+        self.__now_playing_box.set_action("Loaded:")
 
 
     def handle_MEDIA_ACT_PREVIOUS(self):
