@@ -4,6 +4,6 @@ def get_classes():
     from Preferences import Preferences
     from GenericThumbnailer import GenericThumbnailer
     return [Navigator,
-            Preferences,
+            #Preferences,
             GenericThumbnailer]
 
