@@ -1,5 +1,5 @@
 def get_classes():
 
     from Prefs import Prefs
-    return [Prefs]
+    return [] #Prefs]
 
