@@ -1,9 +1,9 @@
-from mediabox.TrackList import TrackList
-from HeaderItem import HeaderItem
 from MediaItem import MediaItem
-from SubItem import SubItem
+from ui.itemview.ThumbableGridView import ThumbableGridView
+from ui.dialog import OptionDialog
 from ui.Pixmap import text_extents
 from theme import theme
+from utils.ItemScheduler import ItemScheduler
 from utils import logging
 
 import time
@@ -15,16 +15,20 @@ _STATUS_INCOMPLETE = 1
 _STATUS_INVALID = 2
 
 
-class StorageBrowser(TrackList):
+class StorageBrowser(ThumbableGridView):
 
     GO_NEW = 0
     GO_CHILD = 1
     GO_PARENT = 2
     
-    EVENT_FOLDER_OPENED = "folder-opened"
+    EVENT_FOLDER_BEGIN = "folder-begin"
+    EVENT_FOLDER_PROGRESS = "folder-progress"
+    EVENT_FOLDER_COMPLETE = "folder-complete"
+
     EVENT_FILE_OPENED = "file-opened"
     EVENT_FILE_ENQUEUED = "file-enqueued"
     EVENT_FILE_REMOVED = "file-removed"
+    EVENT_FILE_BOOKMARKED = "file-bookmarked"
     EVENT_FILE_ADDED_TO_LIBRARY = "file-added-to-library"
     
 
@@ -51,21 +55,35 @@ class StorageBrowser(TrackList):
         # message text to display
         self.__message = ""
         
+        # token for detecting when to abort loading
+        self.__token = 0
+        
         self.__search_term = ""
         
+        self.__tn_scheduler = ItemScheduler()
+        
     
-        TrackList.__init__(self, True)
+        ThumbableGridView.__init__(self)
         self.add_overlay_renderer(self.__render_message)
         self.add_overlay_renderer(self.__render_search_box)
-        self.connect_button_clicked(self.__on_item_button)
-        self.connect_items_swapped(self.__on_swap_items)
+        self.add_overlay_renderer(self.__render_caps)
 
 
-    def connect_folder_opened(self, cb, *args):
+    def connect_folder_begin(self, cb, *args):
     
-        self._connect(self.EVENT_FOLDER_OPENED, cb, *args)
-        
-        
+        self._connect(self.EVENT_FOLDER_BEGIN, cb, *args)
+
+
+    def connect_folder_progress(self, cb, *args):
+    
+        self._connect(self.EVENT_FOLDER_PROGRESS, cb, *args)
+
+
+    def connect_folder_complete(self, cb, *args):
+    
+        self._connect(self.EVENT_FOLDER_COMPLETE, cb, *args)
+
+
     def connect_file_opened(self, cb, *args):
     
         self._connect(self.EVENT_FILE_OPENED, cb, *args)
@@ -81,6 +99,11 @@ class StorageBrowser(TrackList):
         self._connect(self.EVENT_FILE_REMOVED, cb, *args)
 
 
+    def connect_file_bookmarked(self, cb, *args):
+    
+        self._connect(self.EVENT_FILE_BOOKMARKED, cb, *args)
+        
+
     def connect_file_added_to_library(self, cb, *args):
     
         self._connect(self.EVENT_FILE_ADDED_TO_LIBRARY, cb, *args)
@@ -103,7 +126,7 @@ class StorageBrowser(TrackList):
         x, y = self.get_screen_pos()
         w, h = self.get_size()
        
-        tw, th = text_extents(self.__message, theme.font_mb_list_item)
+        tw, th = text_extents(self.__message, theme.font_mb_listitem)
         bw = tw + 20
         bh = th + 6
         tx = (w - bw) / 2 + (bw - tw) / 2
@@ -111,7 +134,7 @@ class StorageBrowser(TrackList):
 
         screen.fill_area((w - bw) / 2, h - bh,
                          bw, bh, theme.color_mb_list_letter_background)
-        screen.draw_text(self.__message, theme.font_mb_list_item, tx, ty,
+        screen.draw_text(self.__message, theme.font_mb_listitem, tx, ty,
                          theme.color_mb_list_letter)
 
 
@@ -130,6 +153,14 @@ class StorageBrowser(TrackList):
                          64, (bh - th) / 2,
                          theme.color_mb_list_letter)
     
+    
+    def __render_caps(self, screen):
+    
+        x, y = self.get_screen_pos()
+        w, h = self.get_size()
+        
+        screen.draw_pixbuf(theme.mb_list_top, 0, 0, w, 32, True)
+        screen.draw_pixbuf(theme.mb_list_bottom, 0, h - 32, w, 32, True)
         
 
 
@@ -139,6 +170,117 @@ class StorageBrowser(TrackList):
         button = item.BUTTON_PLAY
         self.__on_item_button(item, idx, button)
 
+
+    def __on_item_clicked(self, item):
+    
+        idx = self.get_items().index(item)
+    
+        self.set_cursor(idx)
+        self.set_hilight(idx)
+        #self.invalidate_item(idx)
+        self.invalidate()
+        self.render()
+
+        f = item.get_file()
+        
+        def open_file():
+            if (f.mimetype.endswith("-folder")):
+                self.load_folder(f, self.GO_CHILD)
+            else:
+                self.send_event(self.EVENT_FILE_OPENED, f)
+
+        gobject.timeout_add(0, open_file)
+
+
+    def __on_item_menu_opened(self, item):
+    
+        cwd = self.get_current_folder()
+        f = item.get_file()
+        
+        actions = cwd.get_file_actions(f)
+        if (not actions):
+            return
+        
+        dlg = OptionDialog("Options")
+        callbacks = []
+        for icon, name, cb in actions:
+            dlg.add_option(icon, name)
+            callbacks.append(cb)
+           
+        if (dlg.run() != 0):
+            return
+
+        choice = dlg.get_choice()
+        if (choice != -1):
+            callbacks[choice](cwd, f)
+            
+        """
+        idx = self.get_items().index(item)
+    
+        options = []
+
+        if (not f.bookmarked):
+            options.append((None, "Put on Dashboard",
+                            self.EVENT_FILE_BOOKMARKED))
+    
+        if (cwd.folder_flags & cwd.ITEMS_ENQUEUEABLE):
+            options.append((None, "Add to a Playlist",
+                            self.EVENT_FILE_ENQUEUED))
+
+        if (f.folder_flags & f.INDEXABLE):
+            options.append((None, "Add to Media Library",
+                            self.EVENT_FILE_ADDED_TO_LIBRARY))
+
+        if (cwd.folder_flags & cwd.ITEMS_DELETABLE):
+            options.append((None, "Delete",
+                            self.EVENT_FILE_REMOVED))
+
+        if (not options):
+            return
+            
+        dlg = OptionDialog("Options")
+        for icon, label, ev in options:
+            dlg.add_option(icon, label)
+            
+        if (dlg.run() != 0):
+            return
+
+        choice = dlg.get_choice()
+        if (choice != -1):
+            ev = options[choice][2]
+
+            if (ev == self.EVENT_FILE_ENQUEUED):
+                pass
+            
+            elif (ev == self.EVENT_FILE_REMOVED):
+                self.__remove_item(idx)
+            
+            self.emit_event(ev, f)
+        #end if
+        """
+
+
+    """
+    def __on_item_button_pressed(self, button, item):
+    
+        idx = self.get_items().index(item)
+
+        print "BUTTON", button, idx
+        if (button == item.BUTTON_PLAY):
+            f = item.get_file()
+            self.set_hilight(idx)
+            self.render()
+            if (f.mimetype.endswith("-folder")):
+                self.load_folder(f, self.GO_CHILD)
+            else:
+                self.send_event(self.EVENT_FILE_OPENED, f)
+
+        elif (button == item.BUTTON_OPEN):
+            print "OPEN"
+            f = item.get_file()
+            self.insert_folder(f)
+
+
         
     def __on_item_button(self, item, idx, button):
 
@@ -146,7 +288,7 @@ class StorageBrowser(TrackList):
         
         if (button == item.BUTTON_PLAY):
             f = item.get_file()
-            self.hilight(idx)
+            self.set_hilight(idx)
             self.render()
             if (f.mimetype.endswith("-folder")):
                 self.load_folder(f, self.GO_CHILD)
@@ -187,27 +329,23 @@ class StorageBrowser(TrackList):
         elif (button == item.BUTTON_ADD_TO_LIBRARY):
             f = self.get_current_folder()
             self.send_event(self.EVENT_FILE_ADDED_TO_LIBRARY, f)
-
+    """
 
     def __remove_item(self, idx):
 
         folder = self.get_current_folder()
-        # support legacy plugins
-        #f = self.get_items()[idx + 1].get_file()
-        #folder._LEGACY_SUPPORT_file_to_delete = f
-
-        # delete file
         folder.delete_file(idx)
-        #self.send_event(self.EVENT_FILE_REMOVED, f)
 
 
+    """
     def __on_swap_items(self, idx1, idx2):
     
         folder = self.get_current_folder()
         if (folder):
             # -1 because there's a header item
             folder.swap(idx1 - 1, idx2 - 1)
-        
+    """
+
         
     def set_root_device(self, device):
         """
@@ -305,7 +443,7 @@ class StorageBrowser(TrackList):
         @return: list of File objects
         """
     
-        return [ i.get_file() for i in self.get_items()[1:] ]
+        return [ i.get_file() for i in self.get_items() ]
 
 
     def search(self, key, direction):
@@ -362,17 +500,54 @@ class StorageBrowser(TrackList):
         self.__hilighted_file = f
         if (f):
             try:
-                idx = self.get_files().index(f) + 1
+                idx = self.get_files().index(f)
             except ValueError:
                 idx = -1
         else:
             idx = -1
         
-        self.hilight(idx)
+        self.set_hilight(idx)
+        self.invalidate()
         if (idx != -1):
             self.scroll_to_item(idx)
         
         
+    def move_cursor(self, direction, skip_letter):
+        """
+        Moves the cursor by one in the given direction. If C{skip_letter} is
+        C{True}, the cursor skips to the next or previous letter, depending
+        on C{direction}.
+        @since: 0.97
+        
+        @param direction: C{-1} for moving backward or C{1} for moving forward
+        @param skip_letter: whether to skip the current letter
+        """
+        
+        cursor = self.get_cursor()
+        letter = self.get_item(cursor).get_letter()
+        l = len(self.get_items())
+        while (self.get_item(cursor).get_letter() == letter):
+            if (direction == -1):
+                if (cursor > 0):
+                    cursor -= 1
+                else:
+                    break
+            elif (direction == 1):
+                if (cursor + 1 < l):
+                    cursor += 1
+                else:
+                    break
+            #end if
+            
+            if (not skip_letter):
+                break
+            else:
+                self.show_letter()
+        #end while
+
+        self.set_cursor(cursor)
+        
+        
         
     def go_root(self):
         """
@@ -381,15 +556,15 @@ class StorageBrowser(TrackList):
         
         while (len(self.__path_stack) > 1):
             self.__path_stack.pop()
-            self.reload_current_folder()
+        self.reload_current_folder()
 
         
     def go_parent(self):
     
-        if (self.__subfolder_range):
-            self.__close_subfolder()
+        #if (self.__subfolder_range):
+        #    self.__close_subfolder()
         
-        elif (len(self.__path_stack) > 1):
+        if (len(self.__path_stack) > 1):
             self.__path_stack.pop()
             path, status = self.__path_stack.pop()
             if (status == _STATUS_INVALID):
@@ -397,6 +572,7 @@ class StorageBrowser(TrackList):
             else:
                 self.load_folder(path, self.GO_PARENT, False)
 
+
     def reload_current_folder(self):
         """
         Reloads the current folder.
@@ -414,7 +590,7 @@ class StorageBrowser(TrackList):
         @param force_reload: whether to force reloading the folder
         """
 
-        self.__close_subfolder()
+        self.emit_event(self.EVENT_FOLDER_BEGIN, folder)
 
         # are we just reloading the same folder?
         reload_only = False
@@ -434,6 +610,7 @@ class StorageBrowser(TrackList):
             full_reload = True
         elif (direction == self.GO_PARENT):
             full_reload = False
+            #self.__path_stack[-1][1] = _STATUS_INVALID
         else:
             # what else..?
             full_reload = True
@@ -442,32 +619,34 @@ class StorageBrowser(TrackList):
         if (not reload_only):
             self.__path_stack.append([folder, _STATUS_INCOMPLETE])
 
+        self.switch_item_set(folder.full_path)
+        
         if (full_reload):
             # reload list
-            self.change_image_set(folder.full_path)
             self.clear_items()
 
-            header = HeaderItem(folder.name)
-            header.set_info("Retrieving...")
-            buttons = []
-            if (folder.folder_flags & folder.ITEMS_ENQUEUEABLE):
-                buttons.append((header.BUTTON_ENQUEUE, theme.mb_item_btn_enqueue))
-            if (folder.folder_flags & folder.INDEXABLE):
-                buttons.append((header.BUTTON_ADD_TO_LIBRARY, theme.mb_item_btn_add))
-            header.set_buttons(*buttons)
+            #header = HeaderItem(folder.name)
+            #header.set_info("Retrieving...")
+            #buttons = []
+            #if (folder.folder_flags & folder.ITEMS_ENQUEUEABLE):
+            #    buttons.append((header.BUTTON_ENQUEUE, theme.mb_item_btn_enqueue))
+            #if (folder.folder_flags & folder.INDEXABLE):
+            #    buttons.append((header.BUTTON_ADD_TO_LIBRARY, theme.mb_item_btn_add))
+            #header.set_buttons(*buttons)
             
             if (folder.folder_flags & folder.ITEMS_SORTABLE):
                 self.set_drag_sort_enabled(True)
             else:
                 self.set_drag_sort_enabled(False)
 
-            self.append_item(header)
-
         else:
             # don't reload list
-            self.clear_items()
-            self.change_image_set(folder.full_path)
-            self.hilight(-1)
+            pass
+
+        if (folder.folder_flags & folder.ITEMS_COMPACT):
+            self.set_items_per_row(3)
+        else:
+            self.set_items_per_row(1)
 
         # animate
         if (direction == self.GO_CHILD):
@@ -475,73 +654,20 @@ class StorageBrowser(TrackList):
         elif (direction == self.GO_PARENT):
             self.fx_slide_right()
         else:
-            self.render()
+            #self.render()
+            pass
 
         # load remaining items
         loading_status = self.__path_stack[-1][1]
         if (loading_status == _STATUS_INCOMPLETE or full_reload):
             self.set_message("Loading")
             self.complete_current_folder()
+        else:
+            self.emit_event(self.EVENT_FOLDER_COMPLETE, folder)
 
         # now is a good time to collect garbage
         import gc; gc.collect()    
-
-
-    def insert_folder(self, folder):
-
-        def on_child(f, path, entries, items_to_thumbnail, insert_at):
-            # abort if the user has changed the directory again
-            if (self.get_current_folder() != path): return False
-
-            if (f):
-                self.__add_file(f, items_to_thumbnail, insert_at + len(entries))
-                entries.append(f)
-
-            else:
-                # finished loading items; now create thumbnails
-                #self.__create_thumbnails(path, items_to_thumbnail)
-            
-                self.send_event(self.EVENT_FOLDER_OPENED, folder)
-
-            now = time.time()
-            self.__subfolder_range = (insert_at, insert_at + len(entries))
-            if (not f or now > self.__last_list_render_time + 1.0):
-                self.__last_list_render_time = now
-                #self.invalidate_buffer()
-                self.render()
-            
-            return True
-    
-    
-        self.__close_subfolder()
-        idx = self.get_files().index(folder)
-    
-        # change item button
-        item = self.get_item(idx + 1)
-        if (folder.folder_flags & folder.ITEMS_ENQUEUEABLE):
-            item.set_buttons(#(item.BUTTON_CLOSE, theme.mb_item_btn_close),
-                             (item.BUTTON_ENQUEUE, theme.mb_item_btn_enqueue))
-
-        folder.get_contents(0, 0, on_child, self.get_current_folder(), [], [],
-                            idx + 1)
-        
-        
-    def __close_subfolder(self):
-
-        if (not self.__subfolder_range): return
-        
-        idx1, idx2 = self.__subfolder_range
-        n = idx2 - idx1
-        for i in range(n):
-            self.remove_item(idx1 + 1)
-        #end for
-        
-        item = self.get_item(idx1)
-        item.set_buttons((item.BUTTON_OPEN, theme.mb_item_btn_open))
-        
-        self.__subfolder_range = None
-        self.render()
-        
+      
         
     def complete_current_folder(self):
         """
@@ -549,208 +675,99 @@ class StorageBrowser(TrackList):
         current file list.
         """
 
-        def on_child(f, path, entries, items_to_thumbnail):
+        def on_child(f, token, path, entries):
             # abort if the user has changed the directory inbetween
-            if (self.get_current_folder() != path): return False
+            #if (self.get_current_folder() != path): return False
+            if (token != self.__token): return False
             
             if (f):
-                self.get_item(0).set_info("%d items" \
-                                          % len(self.get_files()))
+                #self.get_item(0).set_info("%d items" \
+                #                          % len(self.get_files()))
                 self.set_message("Loading (%d items)" % len(self.get_files()))
-                self.invalidate_image(0)
+                #self.invalidate_item(0)
                 entries.append(f)
-                self.__add_file(f, items_to_thumbnail, -1)
+                try:
+                    self.__add_file(f)
+                    self.emit_event(self.EVENT_FOLDER_PROGRESS,
+                                    self.get_current_folder(), f)
+                except:
+                    print logging.stacktrace()
+
+                import gtk
+                gobject.timeout_add(1, lambda : False)
+                gtk.main_iteration(True)
+
 
             else:
-                self.get_item(0).set_info("%d items" % len(self.get_files()))
-                self.invalidate_image(0)
+                #self.get_item(0).set_info("%d items" % len(self.get_files()))
+                #self.invalidate_item(0)
                 self.set_message("")
                 
                 # mark folder as complete
                 self.__path_stack[-1][1] = _STATUS_OK
                 
                 # finished loading items; now create thumbnails
-                if (items_to_thumbnail):
-                    self.__create_thumbnails(path, items_to_thumbnail,
-                                             len(items_to_thumbnail))
-                
-                self.send_event(self.EVENT_FOLDER_OPENED,
+                #self.__tn_scheduler.resume()
+
+                self.invalidate()
+                self.render()
+                self.emit_event(self.EVENT_FOLDER_COMPLETE,
                                 self.get_current_folder())
+                #self.send_event(self.EVENT_FOLDER_OPENED,
+                #                self.get_current_folder())
 
             now = time.time()
             if (not f or now > self.__last_list_render_time + 1.0):
                 self.__last_list_render_time = now
                 #self.invalidate_buffer()
                 self.render()
-                
+
                 if (not f):
                     return False
+            #end if
             
             return True
 
 
         cwd = self.get_current_folder()
         num_of_items = len(self.get_files())
-        cwd.get_contents(num_of_items, 0, on_child, cwd, [], [])
+        cwd.get_contents(num_of_items, 0, on_child, self.__token, cwd, [])
+        self.__token = (self.__token + 1) % 100
         
         
-    def __add_file(self, f, items_to_thumbnail, insert_at):
+    def __add_file(self, f):
 
         """
         Adds the given file item to the list.
         """
 
-        if (insert_at == -1):
-            # look if there's a thumbnail available
-            if (f.icon):
-                thumbnail = f.icon
-            else:
-                thumbnail = self.__thumbnailer(f, None)
+        cwd = self.get_current_folder()
 
-            item = MediaItem(f, thumbnail)
-            # remember for thumbnailing if no thumbnail was found
-            if (not thumbnail and f.mimetype != f.DIRECTORY):
-                items_to_thumbnail.append((item, None, f))
-
-        else:
-            item = SubItem(f)
+        item = MediaItem(f, f.icon or "")
+        item.connect_activated(self.__on_item_clicked, item)
+        item.connect_menu_opened(self.__on_item_menu_opened, item)
 
         # determine available item buttons
         buttons = []
-        
-        cwd = self.get_current_folder()
-        self.__support_legacy_folder_flags(cwd, f)
 
-        if (f.mimetype in ("application/x-bookmarks-folder",
-                           "application/x-music-folder")):
-            buttons.append((item.BUTTON_OPEN, theme.mb_item_btn_open))
-            
-        elif (f.mimetype.endswith("-folder")):
-            buttons.append((item.BUTTON_PLAY, theme.mb_item_btn_open))
-            
+        if (cwd.folder_flags & cwd.ITEMS_COMPACT):
+            item.set_compact(True)        
+
+        if (cwd.folder_flags & cwd.ITEMS_SORTABLE):
+            item.set_grip_visible(True)
         else:
-            buttons.append((item.BUTTON_PLAY, theme.mb_item_btn_play))
-            
-            if (cwd.folder_flags & cwd.ITEMS_ENQUEUEABLE):
-                buttons.append((item.BUTTON_ENQUEUE, theme.mb_item_btn_enqueue))
-        #end if
-            
-        if (cwd.folder_flags & cwd.ITEMS_DELETABLE):
-            buttons.append((item.BUTTON_REMOVE, theme.mb_item_btn_remove))
-
-        if (cwd.folder_flags & cwd.ITEMS_BULK_DELETABLE):
-            buttons.append((item.BUTTON_REMOVE_PRECEDING, theme.mb_item_btn_remove_up))
-            buttons.append((item.BUTTON_REMOVE_SUCCEEDING, theme.mb_item_btn_remove_down))
+            item.set_grip_visible(False)
+    
+        self.append_item(item)
 
 
-        item.set_buttons(*buttons)
+        idx = len(self.get_items()) - 1
 
-        if (insert_at == -1):
-            if (cwd.folder_flags & cwd.ITEMS_SORTABLE):
-                item.set_grip_visible(True)
-            else:
-                item.set_grip_visible(False)
-        
-            self.append_item(item)
+        #if (not cwd.folder_flags & cwd.ITEMS_SORTED_ALPHA):
+        #    item.set_letter(`idx + 1`)
 
-        else:
-            self.insert_item(item, insert_at)
-            
         # hilight currently selected file
         if (self.__hilighted_file == f):
-            idx = len(self.get_items())
-            self.hilight(idx - 1)
-
-        import gtk
-        cnt = 0
-        while (gtk.events_pending() and cnt < 10):
-            gtk.main_iteration(False)
-            cnt += 1
-
-
-    def __create_thumbnails(self, folder, items_to_thumbnail, total):
-        """
-        Creates thumbnails for the given items.
-        """
-
-        def on_loaded(thumbpath, item, tn):
-            if (item):
-                item.set_icon(thumbpath)
-                item.invalidate()
-                self.invalidate_buffer()
-                self.render()
-            
-            if (tn):
-                tn.set_thumbnail(thumbpath)
-                tn.invalidate()
-            
-            # proceed to next thumbnail
-            gobject.idle_add(self.__create_thumbnails, folder,
-                             items_to_thumbnail, total)
-            
-
-        # abort if the user has changed the directory inbetween
-        if (self.get_current_folder() != folder):
-            self.set_message("")
-            return
-        
-        if (not self.is_visible()):
-            self.set_message("")
-            return
-    
-        if (items_to_thumbnail):
-            # hmm, may we want to reorder a bit?
-            if (len(items_to_thumbnail) % 10 == 0 and \
-                  len(items_to_thumbnail) > 5):
-                idx_in_list = self.get_index_at(0) - 1
-                item_in_view = self.get_files()[idx_in_list]
-                cnt = 0
-                for i in items_to_thumbnail:
-                    if (i[2] == item_in_view):
-                        items_to_thumbnail = \
-                                items_to_thumbnail[cnt:cnt + 5] + \
-                                items_to_thumbnail[:cnt] + \
-                                items_to_thumbnail[cnt + 5:]
-                        break
-                    cnt += 1 
-                #end for
-            #end if            
-        
-            item, tn, f = items_to_thumbnail.pop(0)
-
-            # load thumbnail
-            self.set_message("Creating preview (%d of %d)" \
-                             % (total - len(items_to_thumbnail), total))
-            self.__thumbnailer(f, on_loaded, item, None)
-            
-        else:
-            self.set_message("")
-            self.render()
-        #end if
-
-
-    def __support_legacy_folder_flags(self, folder, f):
-        """
-        Translates the legacy flags to the new folder flags.
-        """
-        
-        if (folder.can_skip):
-            folder.folder_flags |= folder.ITEMS_SKIPPABLE
-            
-        if (folder.can_add_to_library):
-            folder.folder_flags |= folder.INDEXABLE
-
-        if (folder.can_add):
-            folder.folder_flags |= folder.ITEMS_ADDABLE
-
-        if (not f): return
-        
-        if (f.can_delete):
-            folder.folder_flags |= folder.ITEMS_DELETABLE
-
-        if (f.can_keep):
-            folder.folder_flags |= folder.ITEMS_DOWNLOADABLE
-
-        if (f.can_download):
-            folder.folder_flags |= folder.ITEMS_DOWNLOADABLE
+            self.set_hilight(idx)
+        self.invalidate_item(idx)
 
