@@ -2,6 +2,7 @@
 Every widget is derived from this base class.
 """
 
+from utils.EventEmitter import EventEmitter
 from utils import logging
 
 import time
@@ -9,7 +10,7 @@ import threading
 import gtk
 
 
-class Widget(object):
+class Widget(EventEmitter):
     """
     Base class for GDK based lightweight widgets.
     """
@@ -17,13 +18,12 @@ class Widget(object):
     EVENT_BUTTON_PRESS = "button-pressed"
     EVENT_BUTTON_RELEASE = "button-released"
     EVENT_MOTION = "motion"
-    EVENT_KEY_PRESS = "key-pressed"
-    EVENT_KEY_RELEASE = "key-released"
+    EVENT_KEY_PRESSED = "key-pressed"
+    EVENT_KEY_RELEASED = "key-released"
     
+    SLIDE_LEFT = 0
+    SLIDE_RIGHT = 1
     
-    # static lock for blocking event handling
-    __events_lock = threading.Event()
-
     # static lock for animations
     __animation_lock = threading.Event()
 
@@ -47,7 +47,6 @@ class Widget(object):
         self.__children = []
         self.__parent = None
     
-        self.__event_handlers = {}
         self.__locked_zone = None
         self.__need_to_check_zones = False
         
@@ -68,6 +67,7 @@ class Widget(object):
         self.__window = None
         self.__instances.append(self)
 
+        EventEmitter.__init__(self)
 
           
     def push_actor(self, w):
@@ -123,13 +123,13 @@ class Widget(object):
         zone = None
         zone_tstamp = 0
         
-       
         if (ev == Widget.EVENT_BUTTON_PRESS):
             for w in self._zones:
                 if (w.is_frozen()):
                     continue
                 #print self._zones[w]
                 window, x, y, w, h, tstamp, cb = self._zones[w]
+
                 if (window != self.get_window()):
                     continue
             #for x, y, w, h, tstamp, cb in self._zones.values():
@@ -166,24 +166,18 @@ class Widget(object):
         @param *args:   variable number of arguments (depending on the
                         event type)
         """
+       
+        self.emit_event(ev, *args)
 
-        if (self.__events_lock.isSet()): return
-        
+        """
         if (ev in (self.EVENT_KEY_PRESS, self.EVENT_KEY_RELEASE) and
               self._input_focus_widget and
               self._input_focus_widget.is_visible()):
             self._input_focus_widget.send_event(ev, *args)
         
         else:
-            handlers = self.__event_handlers.get(ev, [])
-            for cb, user_args in handlers:
-                try:
-                    cb(*(args + user_args))
-                except:
-                    import traceback; traceback.print_exc()
-            #end for
-        
-        #end if    
+            self.emit_event(ev, *args)
+        """
         
         
     def __on_action(self, etype, px, py):
@@ -216,23 +210,8 @@ class Widget(object):
         """
         
         return self.__animation_lock.isSet()
-        
 
 
-    def set_events_blocked(self, value):
-        """
-        Sets the global lock for blocking events. While events are blocked,
-        no event handling takes place by the Widget class.
-        
-        @param value: whether events are blocked (True) or not (False)
-        """
-    
-        if (value):
-            self.__events_lock.set()
-        else:
-            self.__events_lock.clear()
-    
-    
     def get_children(self):
         """
         Returns a list of all child widgets of this widget.
@@ -432,9 +411,7 @@ class Widget(object):
                 f(c)
     
         self.__is_visible = value
-
         self.__check_zones()
-            
         f(self)
 
 
@@ -501,9 +478,10 @@ class Widget(object):
     def __check_zone(self):
     
         # don't check zone when widget does not have event handlers
-        if (not self.__event_handlers): return
+        #if (not self.__event_handlers): return
+        if (not self.has_events()): return
     
-        if (self.is_enabled() and self.__event_handlers):
+        if (self.is_enabled()):
             x, y = self.get_screen_pos()
             w, h = self.get_size()
             self.__set_zone(self, x, y, w, h)
@@ -525,16 +503,7 @@ class Widget(object):
         and should only be used when implementing new widgets.
         """
     
-        if (not etype in self.__event_handlers):
-            self.__event_handlers[etype] = []
-            
-        # don't allow the same callback twice
-        for c, a in self.__event_handlers[etype]:
-            if (c == cb):
-                return
-        #end for
-            
-        self.__event_handlers[etype].append((cb, args))
+        EventEmitter._connect(self, etype, cb, *args)
         self.__check_zone()
 
 
@@ -600,7 +569,7 @@ class Widget(object):
         @param *args: variable list of user arguments
         """
     
-        self._connect(self.EVENT_KEY_PRESS,
+        self._connect(self.EVENT_KEY_PRESSED,
                       lambda key,*a:cb(key, *a),
                       *args)
                       
@@ -613,7 +582,7 @@ class Widget(object):
         @param *args: variable list of user arguments
         """
     
-        self._connect(self.EVENT_KEY_RELEASE,
+        self._connect(self.EVENT_KEY_RELEASED,
                       lambda key,*a:cb(key, *a),
                       *args)
 
@@ -655,7 +624,7 @@ class Widget(object):
         else:
             parx, pary = (0, 0)
         x, y = self.get_pos()
-        
+
         return (parx + x, pary + y)
         
         
@@ -907,11 +876,12 @@ class Widget(object):
         delta = 1.0 / float(fps)
         next = time.time()
         while (True):
-            #print next
+            # TODO: this stuff should be put into a function in utils
+            gobject.timeout_add(int(delta*1000), lambda : False)
             while (time.time() < next):
                 cnt = 0
                 while (gtk.events_pending() and cnt < 10):
-                    gtk.main_iteration(False)
+                    gtk.main_iteration(True)
                     cnt += 1
                     
             ret = cb(*args)
@@ -920,3 +890,62 @@ class Widget(object):
             if (not ret): break
         #end for
 
+
+
+    def fx_slide_horizontal(self, buf, x, y, w, h, direction):
+
+        def fx(params):
+            from_x, to_x = params
+            dx = (to_x - from_x) / 3
+
+            if (dx > 0):
+                if (direction == self.SLIDE_LEFT):
+                    screen.move_area(scr_x + x + dx, scr_y + y,
+                                     w - dx, h,
+                                     -dx, 0)
+                    screen.copy_pixmap(buf,
+                                       x + from_x, y,
+                                       scr_x + x + w - dx, scr_y + y,
+                                       dx, h)
+                else:
+                    screen.move_area(scr_x + x, scr_y + y,
+                                     w - dx, h,
+                                     dx, 0)
+                    screen.copy_pixmap(buf,
+                                       x + w - from_x - dx, y,
+                                       scr_x + x, scr_y + y,
+                                       dx, h)
+
+                params[0] = from_x + dx
+                params[1] = to_x
+                return True
+
+            else:
+                dx = to_x - from_x
+                if (direction == self.SLIDE_LEFT):
+                    screen.move_area(scr_x + x + dx, scr_y + y,
+                                     w - dx, h,
+                                     -dx, 0)
+                    screen.copy_pixmap(buf,
+                                       x + from_x, y,
+                                       scr_x + x + w - dx,
+                                       scr_y + y,
+                                       dx, h)
+                else:
+                    screen.move_area(scr_x + x, scr_y + y,
+                                     w - dx, h,
+                                     dx, 0)
+                    screen.copy_pixmap(buf,
+                                       x + w - from_x - dx, y,
+                                       scr_y + x, scr_y + y,
+                                       dx, h)
+                
+                return False
+
+
+        if (not self.may_render()): return
+
+        scr_x, scr_y = self.get_screen_pos()
+        screen = self.get_screen()
+        self.animate(50, fx, [0, w])
+
