@@ -44,7 +44,8 @@ class Window(Widget):
                 self.__window.set_default_size(800, 480)
             
         else:
-            self.__window = gtk.Window(gtk.WINDOW_POPUP)
+            self.__window = gtk.Dialog()
+
 
         self.__window.connect("configure-event", self.__on_configure)
         self.__window.connect("expose-event", self.__on_expose)
@@ -69,6 +70,7 @@ class Window(Widget):
         self.__window.set_app_paintable(True)
         self.__window.realize()
 
+        self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
 
         if (platforms.PLATFORM in (platforms.MAEMO5, platforms.MER)):
             import hildon
@@ -81,7 +83,10 @@ class Window(Widget):
 
         self.__fixed = gtk.Fixed()
         self.__fixed.show()
-        self.__window.add(self.__fixed)
+        if (is_toplevel):
+            self.__window.add(self.__fixed)
+        else:
+            self.__window.vbox.add(self.__fixed)
         self.__screen = None
 
         # hide mouse cursor
@@ -92,7 +97,7 @@ class Window(Widget):
             self.__window.window.set_cursor(csr)
             del pbuf
         #end if
-                
+
         self.set_window(self)
 
 
@@ -169,7 +174,7 @@ class Window(Widget):
         else:
             key = gtk.gdk.keyval_name(keyval)
 
-        self.send_event(self.EVENT_KEY_PRESS, key)        
+        self.emit_event(self.EVENT_KEY_PRESSED, key)
         
         # kill queued events
         if (key in ["Up", "Down", "Left", "Right"]):
@@ -182,6 +187,9 @@ class Window(Widget):
 
     def __on_key_released(self, src, ev):
 
+        return True
+
+    """
         keyval = ev.keyval
         c = gtk.gdk.keyval_to_unicode(keyval)
         if (c > 31):
@@ -192,6 +200,7 @@ class Window(Widget):
         self.send_event(self.EVENT_KEY_RELEASE, key)
                 
         return True
+    """
 
 
     def connect_closed(self, cb, *args):
@@ -264,6 +273,11 @@ class Window(Widget):
         self.__window.fullscreen()
 
 
+    def unfullscreen(self):
+    
+        self.__window.unfullscreen()
+
+
     def iconify(self):
     
         self.__window.iconify()
